# encoding:utf-8

from django import forms
from django.forms.widgets import ClearableFileInput, CheckboxInput
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe

from .utils import basename, get_main_model_profile
from .models import Topic, Comment
from .widgets import TextareaWidget


class FormAdminTopic(forms.ModelForm):

    '''
    Form for topic cadmin
    '''
    class Meta:
        model = Topic
        exclude = ('slug', 'id_attachment')
        widgets = {
            'description': TextareaWidget,
        }


class FormAddTopic(forms.ModelForm):

    '''
    Form for create one new topic
    '''

    class Meta:
        model = Topic
        exclude = (
            'forum', "user", "slug", "date",
            "id_attachment", "moderate", "is_top")
        widgets = {
            'description': TextareaWidget,
        }

    def __init__(self, *args, **kwargs):

        super(FormAddTopic, self).__init__(*args, **kwargs)
        class_css = 'form-control'

        for key in self.fields:
            if key != "attachment":
                self.fields[key].required = True
                self.fields[key].widget.attrs['ng-model'] = key
                self.fields[key].widget.attrs['class'] = class_css
                self.fields[key].widget.attrs['required'] = 'required'
            else:
                self.fields[key].required = False


class CustomClearableFileInput(ClearableFileInput):

    '''
      Changes order fields
    '''
    template_with_initial = (
        '%(initial_text)s: <a href="%(initial_url)s">%(initial)s</a> '
        '%(clear_template)s<br />%(input_text)s: %(input)s'
    )
    template_with_clear = ""
    template_with_clear = '<br>  <label for="%(clear_checkbox_id)s"> '
    template_with_clear += ' %(clear_checkbox_label)s</label> %(clear)s'

    def render(self, name, value, attrs=None):
        substitutions = {
            'initial_text': self.initial_text,
            'input_text': self.input_text,
            'clear_template': '',
            'clear_checkbox_label': self.clear_checkbox_label,
        }
        template = '%(input)s'
        substitutions['input'] = super(
            ClearableFileInput, self).render(name, value, attrs)

        if self.is_initial(value):
            template = self.template_with_initial
            substitutions.update(self.get_template_substitution_values(value))

            values = self.get_template_substitution_values(value)
            initial = basename(values['initial'])

            if not self.is_required:
                checkbox_name = self.clear_checkbox_name(name)
                checkbox_id = self.clear_checkbox_id(checkbox_name)

                substitutions['clear_checkbox_name'] = conditional_escape(
                    checkbox_name)

                substitutions['clear_checkbox_id'] = conditional_escape(
                    checkbox_id)

                substitutions['clear'] = CheckboxInput().render(
                    checkbox_name, False, attrs={'id': checkbox_id})

                clear_template = self.template_with_clear % substitutions
                substitutions['clear_template'] = clear_template
                substitutions['initial'] = conditional_escape(initial)

        return mark_safe(template % substitutions)


class FormEditTopic(forms.ModelForm):

    '''
    Form for edit one new topic
    '''

    class Meta:
        model = Topic
        exclude = (
            'forum', "user", "slug", "date",
            "id_attachment", "moderate", "is_top")
        widgets = {
            'description': TextareaWidget,
            'attachment': CustomClearableFileInput,
        }

    def __init__(self, *args, **kwargs):

        super(FormEditTopic, self).__init__(*args, **kwargs)
        class_css = 'form-control'

        for key in self.fields:
            if key != "attachment":
                self.fields[key].required = True
                self.fields[key].widget.attrs['ng-model'] = key
                if key == 'title':
                    ng_init = key + "=" + "'" + \
                        str(kwargs['instance'].title) + "'"
                    self.fields[key].widget.attrs['ng-init'] = ng_init
                elif key == 'description':
                    ng_init = key + "=" + "'" + \
                        str(kwargs['instance'].description) + "'"
                    self.fields[key].widget.attrs['ng-init'] = ng_init
                self.fields[key].widget.attrs['class'] = class_css
                self.fields[key].widget.attrs['required'] = 'required'
            else:
                self.fields[key].required = False


class FormAddComment(forms.ModelForm):

    '''
    Form for add comment to topic
    '''
    class Meta:
        model = Comment
        fields = ['description']
        widgets = {
            'description': TextareaWidget,
        }

    def __init__(self, *args, **kwargs):

        super(FormAddComment, self).__init__(*args, **kwargs)

        for key in self.fields:
            if key == "description":
                self.fields[key].required = True
                self.fields[key].widget.attrs['style'] = "width: 100%"
                self.fields[key].widget.attrs['ng-model'] = key
                self.fields[key].widget.attrs['required'] = 'required'


class FormEditProfile(forms.ModelForm):

    '''
    Form for edit one profile
    '''

    class Meta:
        model = get_main_model_profile()
        exclude = (
            'idprofile', "iduser")
        widgets = {
            'about': TextareaWidget,
            'photo': CustomClearableFileInput,
        }

    def __init__(self, *args, **kwargs):

        super(FormEditProfile, self).__init__(*args, **kwargs)
        class_css = 'form-control'

        for key in self.fields:
            self.fields[key].widget.attrs['class'] = class_css



import os

from django.conf import settings
from django.contrib.auth.models import User, Permission
from django.db import models
from django.shortcuts import get_object_or_404
from django.template import defaultfilters
from django.utils.encoding import python_2_unicode_compatible
from django.utils.crypto import get_random_string
from django.utils.translation import ugettext_lazy as _

from .validators import valid_extension


@python_2_unicode_compatible
class Category(models.Model):

    idcategory = models.AutoField(primary_key=True)
    name = models.CharField(max_length=80)
    position = models.IntegerField(blank=True, default=0)
    hidden = models.BooleanField(
        blank=False, null=False, default=False,
        help_text=_('If checked, this category will be visible only for staff')
    )

    class Meta(object):
        ordering = ['position']
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Forum(models.Model):

    idforum = models.AutoField(primary_key=True)
    category = models.ForeignKey(
        Category, related_name='forums',
        verbose_name=_('Category')
    )
    parent = models.ForeignKey(
        'self', related_name='child_forums', verbose_name=_('Parent forum'),
        blank=True, null=True
    )
    name = models.CharField(_('Name'), max_length=80, unique=True)
    position = models.IntegerField(_('Position'), blank=True, default=0)
    description = models.TextField(_('Description'), blank=True)
    moderators = models.ForeignKey(
        User, blank=True, null=True,
        verbose_name=_('Moderators')
    )
    date = models.DateTimeField(_('Date'), blank=True, null=True)
    topics_count = models.IntegerField(
        _('Topics count'), blank=True, default=0)
    hidden = models.BooleanField(
        _('Hidden'), blank=False, null=False, default=False
    )
    is_moderate = models.BooleanField(_('Check topics'), default=False)

    class Meta(object):
        ordering = ['category', 'position']
        verbose_name = _('Forum')
        verbose_name_plural = _('Forums')

    def __init__(self, *args, **kwargs):
        super(Forum, self).__init__(*args, **kwargs)
        self.old_moderators = self.moderators

    def __str__(self):
        return self.name

    # Return forums that moderating one moderator
    def tot_forums_moderators(self, moderator):
        tot = self.__class__.objects.filter(
            moderators=moderator
        ).count()

        return tot

    def delete(self, *args, **kwargs):
        if not self.moderators.is_superuser:
            if self.moderators:
                # Only remove permissions if is moderator one forum
                tot_forum_moderator = self.tot_forums_moderators(self.moderators)
                if tot_forum_moderator <= 1:
                    # Remove permissions to user
                    try:
                        u = User.objects.get(username=self.moderators)
                        u.user_permissions.clear()
                    except Exception:
                        pass
        super(Forum, self).delete()

    def save(self, *args, **kwargs):
        try:
            if not self.moderators.is_superuser:
                # Remove last moderator
                if self.old_moderators:

                    # Only remove permissions if is moderator one forum
                    tot_forum_moderator = self.tot_forums_moderators(self.old_moderators)
                    if tot_forum_moderator <= 1:
                        u = User.objects.get(username=self.old_moderators)
                        u.user_permissions.clear()

                # Add permissions to user
                u = User.objects.get(username=self.moderators)

                permission1 = Permission.objects.get(codename='add_topic')
                permission2 = Permission.objects.get(codename='change_topic')
                permission3 = Permission.objects.get(codename='delete_topic')

                u.user_permissions.add(permission1)
                u.user_permissions.add(permission2)
                u.user_permissions.add(permission3)
        except Exception:
            pass

        super(Forum, self).save(*args, **kwargs)

    def clean(self):
        if self.name:
            self.name = self.name.strip()

    def escape_html_description(obj):
        return obj.description
    escape_html_description.allow_tags = True


def generate_path(instance, filename):

    folder = ""
    folder = "forum_" + str(instance.forum_id)
    folder = folder + "_user_" + str(instance.user)
    folder = folder + "_topic_" + str(instance.id_attachment)
    return os.path.join("forum", folder, filename)


@python_2_unicode_compatible
class Topic(models.Model):

    idtopic = models.AutoField(primary_key=True)
    forum = models.ForeignKey(
        Forum, related_name='topic', verbose_name=_('Forum')
    )
    user = models.ForeignKey(
        User, related_name='Topic', verbose_name=_('User'))
    slug = models.SlugField(max_length=100)
    title = models.CharField(_('Title'), max_length=80)
    date = models.DateTimeField(_('Date'), blank=False, db_index=False)
    description = models.TextField(_('Description'), blank=False, null=False)
    id_attachment = models.CharField(max_length=200, null=True, blank=True)
    attachment = models.FileField(
        _('File'), blank=True, null=True, upload_to=generate_path,
        validators=[valid_extension]
    )
    moderate = models.BooleanField(_('Moderate'), default=False)
    is_top = models.BooleanField(_('Top'), default=False)

    class Meta(object):
        ordering = ['forum', 'date', 'title']
        verbose_name = _('Topic')
        verbose_name_plural = _('Topics')

    def __str__(self):
        return self.title

    def delete(self, *args, **kwargs):
        idtopic = self.idtopic
        forum = self.forum_id

        topic = get_object_or_404(Topic, idtopic=idtopic)

        folder = ""
        folder = "forum_" + str(forum)
        folder = folder + "_user_" + str(topic.user.username)
        folder = folder + "_topic_" + str(topic.id_attachment)
        path_folder = os.path.join("forum", folder)
        media_path = settings.MEDIA_ROOT
        path = media_path + "/" + path_folder

        # Remove attachment if exists
        from .utils import remove_folder, exists_folder
        if exists_folder(path):
            remove_folder(path)

        Topic.objects.filter(idtopic=idtopic).delete()
        self.update_forum_topics(self.forum, "subtraction")

    def save(self, *args, **kwargs):

        if not self.idtopic:
            self.slug = defaultfilters.slugify(self.title)
            self.update_forum_topics(self.forum, "sum")

        self.generate_id_attachment(self.id_attachment)
        super(Topic, self).save(*args, **kwargs)

    def update_forum_topics(self, forum, action):

        f = Forum.objects.get(name=forum)
        tot_topics = f.topics_count
        if action == "sum":
            tot_topics = tot_topics + 1
        elif action == "subtraction":
            tot_topics = tot_topics - 1

        Forum.objects.filter(name=forum).update(
            topics_count=tot_topics
        )

    def generate_id_attachment(self, value):
        if not value:
            self.id_attachment = get_random_string(length=32)


@python_2_unicode_compatible
class Comment(models.Model):

    idcomment = models.AutoField(primary_key=True)
    topic = models.ForeignKey(
        Topic, related_name='comments', verbose_name=_('Topic')
    )
    user = models.ForeignKey(
        User, related_name='comments', verbose_name=_('User')
    )
    date = models.DateTimeField(_('Date'), blank=True, db_index=True)
    description = models.TextField(_('Description'), blank=True)

    class Meta(object):
        ordering = ['date']
        verbose_name = _('Comment')
        verbose_name_plural = _('Comments')

    def __str__(self):
        return str(self.description)


@python_2_unicode_compatible
class Notification(models.Model):

    idnotification = models.AutoField(primary_key=True)
    idobject = models.IntegerField(default=0)
    iduser = models.IntegerField(default=0)
    is_topic = models.BooleanField(default=0)
    is_comment = models.BooleanField(default=0)
    is_view = models.BooleanField(default=0)
    date = models.DateTimeField(blank=True, db_index=True)

    class Meta(object):
        ordering = ['date']

    def __str__(self):
        return str(self.idnotification)


@python_2_unicode_compatible
class Register(models.Model):

    idregister = models.AutoField(primary_key=True)
    forum = models.ForeignKey(
        Forum, related_name='register', verbose_name=_('Forum')
    )
    user = models.ForeignKey(
        User, related_name='register', verbose_name=_('User')
    )
    date = models.DateTimeField(_('Date'), blank=True, db_index=True)

    class Meta(object):
        ordering = ['date']
        verbose_name = _('Register')
        verbose_name_plural = _('Registers')

    def __str__(self):
        return str(self.forum) + " " + str(self.user)


def generate_path_profile(instance, filename):
    return os.path.join(
        "profiles", "profile_" + str(instance.iduser_id), filename
    )

@python_2_unicode_compatible
class AbstractProfile(models.Model):

    idprofile = models.AutoField(primary_key=True, unique=True)
    iduser = models.OneToOneField(User, db_index=True)
    photo = models.FileField(
                upload_to=generate_path_profile, null=True, blank=True,
            )
    about = models.TextField(blank=True, null=True)

    def __str__(self):
        return str(self.iduser.username)

    class Meta:
        abstract = True



from django.conf import settings
from django.conf.urls import include, url
from django.contrib import admin
from django.contrib.auth.decorators import login_required

from .api import router
from .feeds import TopicFeed
from .views import (
    ForumsView, ForumView, TopicView, NewTopicView,
    EditTopicView, DeleteTopicView, NewCommentView,
    EditCommentView, DeleteCommentView, AllNotification,
    SetNotifications, AddRegisterView, UnregisterView,
    UsersForumView, TopicSearch, ProfileView, EditProfileView
)


admin.site.site_header = settings.SITE_NAME

urlpatterns = [
    #Url for django-rest-framework
    url(r'^', include(router.urls)),
    url(r'^api/', include('rest_framework.urls', namespace='rest_framework')),
    # Url for django-hitcount
    url(r'hitcount/', include('hitcount.urls', namespace='hitcount')),

    # Url's Django-musette
    url(
        r'^forums/$', ForumsView.as_view(), name='forums'
    ),
    url(
        r'^forum/(?P<forum>.+)/$', ForumView.as_view(), name='forum'
    ),
    url(
        r'^topic/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/$',
        TopicView.as_view(), name='topic'
    ),
    url(
        r'^newtopic/(?P<forum>.+)/$', login_required(NewTopicView.as_view()),
        name='newtopic'
    ),
    url(
        r'^edit_topic/(?P<forum>.+)/(?P<idtopic>\d+)/$',
        login_required(EditTopicView.as_view()), name='edittopic'
    ),
    url(
        r'^delete_topic/(?P<forum>.+)/(?P<idtopic>\d+)/$',
        login_required(DeleteTopicView.as_view()), name='deletetopic'
    ),
    url(
        r'^newcomment/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/$',
        login_required(NewCommentView.as_view()), name='newcomment'
    ),
    url(
        r'^updatecomment/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/(?P<idcomment>\d+)/$',
        login_required(EditCommentView.as_view()), name='updatecomment'
    ),
    url(
        r'^removecomment/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/(?P<idcomment>\d+)/$',
        login_required(DeleteCommentView.as_view()), name='removecomment'
    ),
    url(
        r'^forum_all_notification/$', login_required(
            AllNotification.as_view()),
        name='forum_all_notification'
    ),
    url(
        r'^forum_set_notifications/$', login_required(SetNotifications),
        name='forum_set_notifications'
    ),
    url(
        r'^new_register/(?P<forum>.+)/$', login_required(
            AddRegisterView.as_view()),
        name='new_register'
    ),
    url(
        r'^unregister/(?P<forum>.+)/$', login_required(UnregisterView.as_view()
                                                       ),
        name='unregister'
    ),
    url(
        r'^users_forum/(?P<forum>.+)/$', UsersForumView.as_view(),
        name='users_forum'
    ),
    url(
        r'^search_topic/(?P<forum>.+)/$', TopicSearch.as_view(),
        name='search_topic'
    ),
    url(
        r'^feed/(?P<forum>.+)/$', TopicFeed(), name='rss'
    ),
    url(
        r'^profile/(?P<username>.+)/$', ProfileView.as_view(), name='profile'
    ),
    url(
        r'^edit_profile/(?P<username>[-\w]+)/$', EditProfileView.as_view(), name='edit_profile'
    ),
]



# -*- coding: UTF-8 -*-
import os
import shutil

from django.apps import apps
from django.conf import settings
from django.core.paginator import Paginator
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.shortcuts import get_object_or_404
from django.utils import formats, timezone
from django.utils.translation import ugettext_lazy as _

from .models import (
    Forum, Topic, Comment, 
    Notification, AbstractProfile
)


def exists_folder(route):
    '''
        This method verify that exists
        folder in base to route
    '''
    if os.path.exists(route):
        return True
    else:
        return False


def remove_folder(route_folder):
    '''
        This method remove one folder
    '''
    try:
        shutil.rmtree(route_folder)
    except Exception:
        pass


def remove_file(route_file):
    '''
        This method remove one file
        in base to route and image
    '''
    if route_file != "" and not route_file is None:
        if os.path.exists(route_file):
            os.remove(route_file)


def get_folder_attachment(topic):
    '''
        This method return the path of one
        folder attachment for app forum
    '''
    folder = ""
    folder = "forum_" + str(topic.forum_id)
    folder = folder + "_user_" + str(topic.user.username)
    folder = folder + "_topic_" + str(topic.id_attachment)
    path_folder = os.path.join("forum", folder)
    media_path = settings.MEDIA_ROOT
    path = media_path + "/" + path_folder

    return path


def remove_folder_attachment(idtopic):
    '''
        This method remove folder attachment
        and subtract one topic.
    '''
    # Subtract one topic
    topic = get_object_or_404(Topic, idtopic=idtopic)
    forum = get_object_or_404(Forum, name=topic.forum, hidden=False)
    tot = forum.topics_count
    tot = tot - 1
    Forum.objects.filter(name=topic.forum, hidden=False).update(
        topics_count=tot
    )

    path = get_folder_attachment(topic)

    # Remove attachment if exists
    if exists_folder(path):
        remove_folder(path)


def get_main_model_profile():
    '''
        This method return the model profile defined by user
    '''
    subclasses = AbstractProfile.__subclasses__()
    if len(subclasses) > 0:
        try:
            return subclasses[0]
        except Exception:
            raise BaseException("Occurs one error to get model profile") 
    else:
        raise BaseException("It is not defined profile model")


def get_app_model(instance):

    '''
        This method get app_label from model instance
    '''
    return instance._meta.app_label


def get_count_fields_model(instance):

    '''
        This method get count fields from model instance
    '''
    return len(instance._meta.fields)


def get_id_profile(iduser):
    '''
        This method return one id
        of model profile
    '''
    ModelProfile = get_main_model_profile()
    profile = get_object_or_404(ModelProfile, iduser=iduser)

    return profile


def get_users_topic(topic, myuser):
    '''
        This method return all users
        of one topic, else my user
    '''
    comments = Comment.objects.filter(topic_id=topic.idtopic)
    lista_us = []
    for comment in comments:
        if comment.user_id != myuser:
            if not comment.user_id in lista_us:
                lista_us.append(comment.user_id)

    return lista_us


def get_notifications(iduser):
    '''
        This method return Notification
        of one user
    '''
    try:
        notif = Notification.objects.filter(
            iduser=iduser).order_by("-date")
    except Notification.DoesNotExist:
        notif = None

    return notif


def get_datetime_topic(date):
    '''
        This method return info one datetime
        for topic or notification
    '''
    flag = True
    now = timezone.now()
    # If is beteween 1 and 10 return days
    difference = (now - date).days

    # If is this days return hours
    if difference == 0:
        flag = False
        diff = now - date
        minutes = (diff.seconds//60) % 60
        hours = diff.seconds//3600
        if minutes < 60 and hours == 0:
            difference = "%s %s" % (
                str((diff.seconds//60) % 60) + "m ", _("ago"))
        else:
            difference = "%s %s" % (str(diff.seconds//3600) + "h ", _("ago"))

    # If is days
    if flag:
        difference = "%s %s" % (str(difference),  _("days ago"))

    return difference


def basename(value):
    '''
            This method return basename
            of one path
    '''
    return os.path.basename(value)


def helper_paginator(self, request, model, tot_record, nonRecPag):
    '''
        This function is responsible of Pagination
    '''
    result_list = Paginator(model, tot_record)
    try:
        page = int(request.GET.get('page'))
    except:
        page = 1

    if page <= 0:
        page = 1

    if(page > result_list.num_pages):
        page = result_list.num_pages

    if (result_list.num_pages >= page):
        pagina = result_list.page(page)
        Contexto = {
            nonRecPag: pagina.object_list,
            'page': page,
            'pages': result_list.num_pages,
            'has_next': pagina.has_next(),
            'has_prev': pagina.has_previous(),
            'next_page': page+1,
            'prev_page': page-1,
            'firstPage': 1,
        }
        return Contexto


def get_route_file(file_path, file_name):
    '''
        This method build the path for a file MEDIA
    '''
    try:
        route_file = file_path + "/" + file_name
    except Exception:
        route_file = ""

    return route_file


def get_photo_profile(iduser):
    '''
        This method return photo profile
    '''
    default_photo = static("img/profile.png")
    ModelProfile = get_main_model_profile()
    profile = ModelProfile.objects.filter(iduser=iduser)
    if profile.count() > 0:
        photo = profile[0].photo
        if photo:
            field_photo = settings.MEDIA_URL + str(photo)
        else:
            field_photo = default_photo
    else:
        field_photo = default_photo
    return field_photo



from django.db import router
from django.contrib import admin, messages
from django.contrib.admin import helpers
from django.contrib.admin.utils import get_deleted_objects
from django.contrib.auth.models import User
from django.core.exceptions import PermissionDenied
from django.template.response import TemplateResponse
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from .forms import FormAdminTopic
from .models import Category, Forum, Topic, Comment, Register

from .utils import remove_folder_attachment


class TopicAdmin(admin.ModelAdmin):
    form = FormAdminTopic
    list_display = ('title', 'forum', 'date')
    list_filter = ['title', 'date']
    search_fields = ['title']
    actions = ['delete_topic']

    def get_queryset(self, request):
        qs = super(TopicAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs

        forums = Forum.objects.filter(moderators=request.user.id)
        lista = []
        for f in forums:
            lista.append(f.idforum)

        return qs.filter(forum_id__in=lista)

    def get_actions(self, request):
        actions = super(TopicAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']
        return actions

    def delete_topic(self, request, queryset):
        '''
            This method remove topic's selected in the admin django.
            Can remove one o more records.
        '''
        if not self.has_delete_permission(request):
            raise PermissionDenied

        if request.POST.get("post"):
            for obj in queryset:
                idtopic = obj.idtopic
                # Remove folder attachment
                remove_folder_attachment(idtopic)
                # Delete record
                Topic.objects.filter(idtopic=idtopic).delete()

            n = queryset.count()
            self.message_user(
                request,
                _("Successfully deleted %(count)d record/s.") % {
                    "count": n, }, messages.SUCCESS
            )

            return None
        else:

            opts = self.model._meta

            if len(queryset) == 1:
                objects_name = force_text(opts.verbose_name)
            else:
                objects_name = force_text(opts.verbose_name_plural)

            using = router.db_for_write(self.model)

            del_obj, model_c, perms_n, protected = get_deleted_objects(
                queryset, opts, request.user, self.admin_site, using
            )

            context = {
                'title': "",
                'delete_topic': [queryset],
                'ids': queryset.values_list("idtopic"),
                'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
                'opts': opts,
                'objects_name': objects_name,
                'deletable_objects': [del_obj],
            }

            return TemplateResponse(
                request, 'musette/admin/confirm_delete.html',
                context, current_app=self.admin_site.name
            )

    delete_topic.short_description = _(
        "Delete selected %(verbose_name_plural)s"
    )


class ForumAdmin(admin.ModelAdmin):
    list_display = (
        'name', 'category', 'escape_html_description',
        'moderators', 'topics_count')
    list_filter = ['name', 'category']
    search_fields = ['name']
    actions = ['delete_topic']

    def get_actions(self, request):
        actions = super(ForumAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']
        return actions

    def delete_topic(self, request, queryset):
        '''
            This method remove forum selected
            in the admin django. Can remove one
            o more records.
        '''
        if not self.has_delete_permission(request):
            raise PermissionDenied

        if request.POST.get("post"):
            for obj in queryset:
                idforum = obj.idforum

                if not obj.moderators.is_superuser:
                    if obj.moderators:

                        # Return forums that moderating one moderator
                        tot_forum_moderator = Forum.objects.filter(
                            moderators=obj.moderators).count()

                        # Only remove permissions if is moderator one forum
                        if tot_forum_moderator <= 1:
                            try:
                                u = User.objects.get(username=obj.moderators)
                                u.user_permissions.clear()
                            except Exception:
                                pass

                # Delete record
                Forum.objects.filter(idforum=idforum).delete()

            n = queryset.count()
            self.message_user(
                request,
                _("Successfully deleted %(count)d record/s.") % {
                    "count": n, }, messages.SUCCESS
            )

            return None
        else:

            opts = self.model._meta

            if len(queryset) == 1:
                objects_name = force_text(opts.verbose_name)
            else:
                objects_name = force_text(opts.verbose_name_plural)

            using = router.db_for_write(self.model)

            del_obj, model_c, perms_n, protected = get_deleted_objects(
                queryset, opts, request.user, self.admin_site, using
            )

            context = {
                'title': "",
                'delete_topic': [queryset],
                'ids': queryset.values_list("idforum"),
                'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
                'opts': opts,
                'objects_name': objects_name,
                'deletable_objects': [del_obj],
            }

            return TemplateResponse(
                request, 'musette/admin/confirm_delete.html',
                context, current_app=self.admin_site.name
            )

    delete_topic.short_description = _(
        "Delete selected %(verbose_name_plural)s"
    )

    # TinyMCE
    class Media:
        js = ('/static/js/libs/tiny_mce/tiny_mce.js',
              '/static/js/textareas.js')


class CommentAdmin(admin.ModelAdmin):
    list_display = (
        'topic', 'forum', 'user')
    list_filter = ['topic', 'user']
    search_fields = ['topic', 'user']

    def forum(self, obj):
        return obj.topic.forum
    forum.short_description = 'Forum'
    forum.admin_order_field = 'topic__forum'


admin.site.register(Category)
admin.site.register(Forum, ForumAdmin)
admin.site.register(Topic, TopicAdmin)
admin.site.register(Comment, CommentAdmin)
admin.site.register(Register)



import os

from django import template
from django.template.defaultfilters import stringfilter

register = template.Library()


@register.filter(name='basename')
@stringfilter
def basename(value):
    return os.path.basename(value)



# encoding:utf-8
from django import template
from django.shortcuts import get_object_or_404

from ..utils import get_photo_profile

register = template.Library()


@register.filter
def get_photo(user):
    '''
    This tag return the path photo profile
    '''
    field_photo = get_photo_profile(user)
    return field_photo



# -*- coding: UTF-8 -*-
import datetime
import json
import redis

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.models import User
from django.core.mail import send_mail
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404, redirect
from django.template import defaultfilters
from django.views.generic import View
from django.views.generic.edit import FormView
from django.utils.crypto import get_random_string
from django.utils.html import strip_tags
from django.utils.text import Truncator
from django.utils.translation import ugettext_lazy as _

from log.utils import set_error_to_log

from .forms import (
    FormAddTopic, FormEditTopic, 
    FormAddComment, FormEditProfile
)
from .models import (
    Category, Forum, Topic, 
    Comment, Notification, Register,
    AbstractProfile
)
from .utils import (
    remove_folder_attachment, get_id_profile,
    get_users_topic, exists_folder,
    get_notifications, remove_file,
    get_route_file, remove_folder,
    get_photo_profile, get_main_model_profile,
    get_app_model, get_count_fields_model
)


class ForumsView(View):

    '''
        This view display all forum registered
    '''
    template_name = "musette/index.html"

    def get(self, request, *args, **kwargs):

        categories = Category.objects.filter(hidden=False)

        data = {
            'categories': categories
        }

        return render(request, self.template_name, data)


class ForumView(View):

    '''
        This view display one forum registered
    '''

    def get(self, request, forum, *args, **kwargs):

        template_name = "musette/forum_index.html"
        page_template = "musette/forum.html"

        forum = get_object_or_404(Forum, name=forum, hidden=False)
        topics = Topic.objects.filter(
            forum_id=forum.idforum).order_by("-is_top", "-date")

        iduser = request.user.id
        if iduser:
            try:
                Register.objects.get(
                    forum_id=forum.idforum, user_id=iduser
                )
                register = True
            except Register.DoesNotExist:
                register = False

        else:
            notifications = None
            register = False

        data = {
            'forum': forum,
            'topics': topics,
            'register': register
        }

        if request.is_ajax():
            template_name = page_template

        return render(request, template_name, data)


class TopicView(View):

    '''
        This view display one Topic of forum
    '''

    def get(self, request, forum, slug, idtopic, *args, **kwargs):

        template_name = "musette/topic_index.html"
        page_template = "musette/topic.html"

        forum = get_object_or_404(Forum, name=forum, hidden=False)
        topic = get_object_or_404(Topic, idtopic=idtopic, slug=slug)

        form_comment = FormAddComment()

        comments = Comment.objects.filter(topic_id=idtopic)

        photo = get_photo_profile(topic.user.id)

        data = {
            'topic': topic,
            'form_comment': form_comment,
            'comments': comments,
            'photo': photo
        }

        if request.is_ajax():
            template_name = page_template
        return render(request, template_name, data)


class NewTopicView(FormView):

    '''
        This view allowed add new topic
    '''
    template_name = "musette/new_topic.html"
    form_class = FormAddTopic

    def get_success_url(self):
        return '/forum/' + self.kwargs['forum']

    def get(self, request, forum, *args, **kwargs):

        data = {
            'form': self.form_class,
            'forum': forum,
        }
        return render(request, self.template_name, data)

    def post(self, request, forum, *args, **kwargs):

        form = FormAddTopic(request.POST, request.FILES)

        if form.is_valid():
            obj = form.save(commit=False)

            now = datetime.datetime.now()
            user = User.objects.get(id=request.user.id)
            forum = get_object_or_404(Forum, name=forum)
            title = strip_tags(request.POST['title'])

            obj.date = now
            obj.user = user
            obj.forum = forum
            obj.title = title
            obj.slug = defaultfilters.slugify(request.POST['title'])

            if 'attachment' in request.FILES:
                id_attachment = get_random_string(length=32)
                obj.id_attachment = id_attachment

                file_name = request.FILES['attachment']
                obj.attachment = file_name

            if forum.is_moderate:
                if forum.moderators_id == request.user.id:
                    obj.moderate = True
                else:
                    obj.moderate = False

                    # Send email to moderator
                    if settings.SITE_URL.endswith("/"):
                        site = settings.SITE_URL + "forum/" + forum.name
                    else:
                        site = settings.SITE_URL + "/forum/" + forum.name

                    title_email = "New notification " + settings.SITE_NAME
                    message = "You have one new topic: " + site
                    email_from = settings.EMAIL_MUSETTE
                    user_moderator = get_object_or_404(User, id=forum.moderators_id)
                    email_moderator = user_moderator.email
                    if email_from:
                        send_mail(title_email, message, email_from, [email_moderator], fail_silently=False)
            else:
                obj.moderate = True

            obj.save()
            return self.form_valid(form, **kwargs)
        else:
            messages.error(request, _("Form invalid"))
            return self.form_invalid(form, **kwargs)


class EditTopicView(FormView):

    '''
        This view allowed edit topic
    '''
    template_name = "musette/edit_topic.html"
    form_class = FormEditTopic

    def get_success_url(self):
        return '/forum/' + self.kwargs['forum']

    def get(self, request, forum, idtopic, *args, **kwargs):

        topic = get_object_or_404(
            Topic, idtopic=idtopic, user_id=request.user.id
        )

        # Init fields form
        form = FormEditTopic(instance=topic)

        data = {
            'form': form,
            'forum': forum,
            'topic': topic,
        }

        return render(request, self.template_name, data)

    def post(self, request, forum, idtopic, *args, **kwargs):

        topic = get_object_or_404(
            Topic, idtopic=idtopic, user_id=request.user.id
        )
        file_name = topic.attachment

        form = FormEditTopic(request.POST, request.FILES, instance=topic)
        file_path = settings.MEDIA_ROOT

        if form.is_valid():

            obj = form.save(commit=False)

            title = strip_tags(request.POST['title'])
            description = strip_tags(request.POST['description'])
            slug = defaultfilters.slugify(request.POST['title'])

            obj.title = title
            obj.description = description
            obj.slug = slug

            # If check field clear, remove file when update
            if 'attachment-clear' in request.POST:
                route_file = get_route_file(file_path, file_name.name)

                try:
                    remove_file(route_file)
                except Exception:
                    pass

            if 'attachment' in request.FILES:

                if not topic.id_attachment:
                    id_attachment = get_random_string(length=32)
                    obj.id_attachment = id_attachment

                file_name_post = request.FILES['attachment']
                obj.attachment = file_name_post

                # Route previous file
                route_file = get_route_file(file_path, file_name.name)

                try:
                    # If a previous file exists it removed
                    remove_file(route_file)
                except Exception:
                    pass

            # Update topic
            form.save()

            return self.form_valid(form, **kwargs)
        else:
            messages.error(request, _("Form invalid"))
            return self.form_invalid(form, **kwargs)


class DeleteTopicView(View):

    '''
        This view will delete one topic
    '''

    def get(self, request, forum, idtopic, *args, **kwargs):

        # Previouly verify that exists the topic
        topic = get_object_or_404(
            Topic, idtopic=idtopic, user_id=request.user.id
        )

        iduser_topic = topic.user_id

        # If my user delete
        if request.user.id == iduser_topic:
            remove_folder_attachment(idtopic)
            Topic.objects.filter(
                idtopic=idtopic, user_id=iduser_topic).delete()
        else:
            error = ""
            error = error + 'The user ' + str(request.user.id)
            error = error + ' He is trying to remove the job ' + str(idtopic)
            error = error + ' of user ' + str(iduser_topic)

            set_error_to_log(request, error)
            raise Http404

        return redirect('forum', forum)


class NewCommentView(View):

    '''
        This view allowed add new comment to topic
    '''

    def get(self, request, forum, slug, idtopic, *args, **kwargs):
        raise Http404()

    def post(self, request, forum, slug, idtopic, *args, **kwargs):

        form = FormAddComment(request.POST)

        param = ""
        param = forum + "/" + slug
        param = param + "/" + str(idtopic) + "/"
        url = '/topic/' + param

        if form.is_valid():
            obj = form.save(commit=False)

            now = datetime.datetime.now()
            user = User.objects.get(id=request.user.id)
            topic = get_object_or_404(Topic, idtopic=idtopic)

            obj.date = now
            obj.user = user
            obj.topic_id = topic.idtopic

            obj.save()

            r = redis.StrictRedis()

            idcomment = obj.idcomment

            # Data for notification real time
            comment = Comment.objects.get(idcomment=idcomment)
            profile = get_id_profile(request.user.id)
            username = request.user.username

            # Get photo profile
            photo = get_photo_profile(request.user.id)

            # Data for notification real time
            description = Truncator(comment.description).chars(100)

            # Send notifications
            lista_us = get_users_topic(topic, request.user.id)
            lista_email = []

            # If not exists user that create topic, add
            user_original_topic = topic.user.id
            user_email = topic.user.email

            if not user_original_topic in lista_us:
                lista_us.append(user_original_topic)
                lista_email.append(user_email)
            else:
                user_original_topic = None

            for user in lista_us:
                if user_original_topic != request.user.id:
                    notification = Notification(
                        iduser=user, is_view=False,
                        idobject=idcomment, date=now,
                        is_topic=False, is_comment=True
                    )
                    notification.save()

            # Send email notification
            if settings.SITE_URL.endswith("/"):
                site = settings.SITE_URL[:-1]
            else:
                site = settings.SITE_URL

            title_email = "New notification " + settings.SITE_NAME
            message = "You have one new comment in the topic: " + site + url
            email_from = settings.EMAIL_MUSETTE
            if email_from:
                send_mail(title_email, message, email_from, lista_email, fail_silently=False)

            data = {
                "description": description,
                "topic": comment.topic.title,
                "idtopic": comment.topic.idtopic,
                "slug": comment.topic.slug,
                "settings_static": settings.STATIC_URL,
                "username": username,
                "forum": forum,
                "lista_us": lista_us,
                "photo": photo
            }

            json_data = json.dumps(data)
            r.publish('notifications', json_data)

            return HttpResponseRedirect(url)
        else:
            messages.error(request, _("Field required"))
            return HttpResponseRedirect(url)


class EditCommentView(View):

    '''
        This view allowed edit comment to topic
    '''

    def get(self, request, forum, slug, idtopic, idcomment, *args, **kwargs):
        raise Http404()

    def post(self, request, forum, slug, idtopic, idcomment, *args, **kwargs):

        param = ""
        param = forum + "/" + slug
        param = param + "/" + str(idtopic) + "/"
        url = '/topic/' + param

        description = request.POST.get('update_description')

        if description:

            iduser = request.user.id
            Comment.objects.filter(idcomment=idcomment, user=iduser).update(
                description=description
            )

            return HttpResponseRedirect(url)
        else:
            return HttpResponseRedirect(url)


class DeleteCommentView(View):

    '''
        This view allowed remove comment to topic
    '''

    def get(self, request, forum, slug, idtopic, idcomment, *args, **kwargs):
        raise Http404()

    def post(self, request, forum, slug, idtopic, idcomment, *args, **kwargs):

        param = ""
        param = forum + "/" + slug
        param = param + "/" + str(idtopic) + "/"
        url = '/topic/' + param

        try:
            iduser = request.user.id
            Comment.objects.filter(idcomment=idcomment, user=iduser).delete()
            Notification.objects.filter(idobject=idcomment).delete()

            return HttpResponseRedirect(url)
        except Exception:
            return HttpResponseRedirect(url)


class AllNotification(View):

    '''
        This view return all notification
        and paginate
    '''

    def get(self, request, *args, **kwargs):

        template_name = "musette/all_notification_index.html"
        page_template = "musette/all_notification.html"

        iduser = request.user.id

        Notification.objects.filter(iduser=iduser).update(is_view=True)

        notifications = get_notifications(iduser)
        data = {
            'notifications': notifications,
        }

        if request.is_ajax():
            template_name = page_template
        return render(request, template_name, data)


def SetNotifications(request):
    '''
        This view set all views notifications in true
    '''
    iduser = request.user.id
    Notification.objects.filter(iduser=iduser).update(is_view=True)

    return HttpResponse("Ok")


class AddRegisterView(View):

    '''
        This view add register to forum
    '''

    def get(self, request, forum, *args, **kwargs):
        raise Http404()

    def post(self, request, forum, *args, **kwargs):

        url = '/forum/' + forum + "/"

        forum = get_object_or_404(Forum, name=forum, hidden=False)
        idforum = forum.idforum
        iduser = request.user.id
        date = datetime.datetime.now()

        register = Register(
            forum_id=idforum, user_id=iduser,
            date=date
        )
        register.save()
        return HttpResponseRedirect(url)


class UnregisterView(View):

    '''
        This view remove register to forum
    '''

    def get(self, request, forum, *args, **kwargs):
        raise Http404()

    def post(self, request, forum, *args, **kwargs):

        url = '/forum/' + forum + "/"

        forum = get_object_or_404(Forum, name=forum, hidden=False)
        idforum = forum.idforum
        iduser = request.user.id

        Register.objects.filter(
            forum_id=idforum, user_id=iduser,
        ).delete()

        return HttpResponseRedirect(url)


class UsersForumView(View):

    '''
        This view display users register in forum
    '''
    def get(self, request, forum, *args, **kwargs):

        template_name = "musette/users_forum_index.html"
        page_template = "musette/users_forum.html"

        forum = get_object_or_404(Forum, name=forum, hidden=False)
        registers = Register.objects.filter(forum_id=forum.idforum)

        data = {
            'forum': forum,
            'registers': registers,
        }

        if request.is_ajax():
            template_name = page_template
        return render(request, template_name, data)

    def post(self, request, forum, *args, **kwargs):
        raise Http404()


class TopicSearch(View):

    '''
        This view django, display results of search of topics
    '''

    def get(self, request, forum, *args, **kwargs):

        template_name = "musette/topic_search_index.html"
        page_template = "musette/topic_search.html"

        search = request.GET.get('q')

        forum = get_object_or_404(Forum, name=forum)
        idforum = forum.idforum

        topics = Topic.objects.filter(
            forum_id=idforum, title__icontains=search
        )

        data = {
            'topics': topics,
            'forum': forum,
        }

        if request.is_ajax():
            template_name = page_template
        return render(request, template_name, data)


class ProfileView(View):

    '''
        This view django, display results of the profile
    '''

    def get(self, request, username, *args, **kwargs):
        template_name = "musette/profile.html"

        user = get_object_or_404(User, username=username)
        iduser = user.id

        # Get model extend Profile
        ModelProfile = get_main_model_profile()

        # Get name app of the extend model Profile
        app = get_app_model(ModelProfile)

        # Check if the model profile is extended
        count_fields_model = get_count_fields_model(ModelProfile)
        count_fields_abstract = get_count_fields_model(AbstractProfile)
        if count_fields_model > count_fields_abstract:
            model_profile_is_extend = True
        else:
            model_profile_is_extend = False
        profile = get_object_or_404(ModelProfile, iduser=iduser)

        photo = get_photo_profile(iduser)

        data = {
            'profile': profile,
            'photo': photo,
            'user': request.user,
            'app': app,
            'model_profile_is_extend': model_profile_is_extend
        }

        return render(request, template_name, data)


class EditProfileView(FormView):

    '''
        This view allowed edit profile
    '''
    template_name = "musette/edit_profile.html"
    form_class = FormEditProfile

    def get_success_url(self):
        return '/profile/' + self.kwargs['username']

    def get(self, request, username, *args, **kwargs):

        ModelProfile = get_main_model_profile()
        profile = get_object_or_404(
            ModelProfile, iduser=request.user.id
        )

        # Init fields form
        form = FormEditProfile(instance=profile)

        data = {
            'form': form
        }

        return render(request, self.template_name, data)

    def post(self, request, username, *args, **kwargs):

        ModelProfile = get_main_model_profile()
        profile = get_object_or_404(
            ModelProfile, iduser=request.user.id
        )

        file_name = profile.photo

        form = FormEditProfile(request.POST, request.FILES, instance=profile)
        file_path = settings.MEDIA_ROOT

        if form.is_valid():

            obj = form.save(commit=False)

            about = strip_tags(request.POST['about'])

            obj.about = about

            # If check field clear, remove file when update
            if 'attachment-clear' in request.POST:
                route_file = get_route_file(file_path, file_name.name)

                try:
                    remove_file(route_file)
                except Exception:
                    pass

            if 'attachment' in request.FILES:

                if not topic.id_attachment:
                    id_attachment = get_random_string(length=32)
                    obj.id_attachment = id_attachment

                file_name_post = request.FILES['photo']
                obj.photo = file_name_post

                # Route previous file
                route_file = get_route_file(file_path, file_name.name)

                try:
                    # If a previous file exists it removed
                    remove_file(route_file)
                except Exception:
                    pass

            # Update profile
            form.save()

            return self.form_valid(form, **kwargs)
        else:
            messages.error(request, _("Form invalid"))
            return self.form_invalid(form, **kwargs)




from __future__ import print_function

import threading
from functools import partial

import redis
import tornado.httpserver
import tornado.ioloop
import tornado.web
import tornado.websocket


LISTENERS = []


def redis_listener():
    r = redis.Redis()
    ps = r.pubsub()
    ps.subscribe('notifications')
    io_loop = tornado.ioloop.IOLoop.instance()
    for message in ps.listen():
        for element in LISTENERS:
            io_loop.add_callback(partial(element.on_message, message))


class RealtimeHandler(tornado.websocket.WebSocketHandler):

    def check_origin(self, origin):
        return True

    def open(self):
        print('New connection was opened')
        LISTENERS.append(self)

    def on_message(self, message):
        self.write_message(message['data'])

    def on_close(self):
        print('Conn closed...')
        LISTENERS.remove(self)


settings = {
    'auto_reload': True,
}

application = tornado.web.Application([
    (r'/ws/', RealtimeHandler),
], **settings)


if __name__ == "__main__":
    threading.Thread(target=redis_listener).start()
    http_server = tornado.httpserver.HTTPServer(application)
    http_server.listen(8888)
    tornado.ioloop.IOLoop.instance().start()



from django import forms


class TextareaWidget(forms.Textarea):

    '''
    Widget rich textarea
    '''
    class Media:
        # tiny_mce
        js = ('/static/js/libs/tiny_mce/tiny_mce.js',
              '/static/js/textareas.js')



from django.contrib.auth.models import User

from rest_framework import serializers, viewsets, routers

from .models import (
    Category, Forum, Topic, 
    Register, Comment
)
from .utils import get_main_model_profile


# Serializers Users
class UserSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = User
        fields = ('url', 'username', 'email', 'is_staff')


# ViewSets for user
class UserViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = User.objects.all()
    serializer_class = UserSerializer


# Serializers Categories
class CategorySerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = Category


# ViewSets for categiry
class CategoryViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Category.objects.all()
    serializer_class = CategorySerializer


# Serializers Forum
class ForumSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = Forum


# ViewSets for forum
class ForumViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Forum.objects.all()
    serializer_class = ForumSerializer


# Serializers Topic
class TopicSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = Topic


# ViewSets for topic
class TopicViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Topic.objects.all()
    serializer_class = TopicSerializer


# Serializers register
class RegisterSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = Register


# ViewSets for register
class RegisterViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Register.objects.all()
    serializer_class = RegisterSerializer


# Serializers comment
class CommentSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = Comment


# ViewSets for comment
class CommentViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Comment.objects.all()
    serializer_class = CommentSerializer


# Serializers profile
class ProfileSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = get_main_model_profile()


# ViewSets for profile
class ProfileViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = get_main_model_profile().objects.all()
    serializer_class = ProfileSerializer


# Routers provide a way of automatically determining the URL conf.
router = routers.DefaultRouter()
router.register(r'api/users', UserViewSet)
router.register(r'api/categories', CategoryViewSet)
router.register(r'api/forums', ForumViewSet)
router.register(r'api/topics', TopicViewSet)
router.register(r'api/registers', RegisterViewSet)
router.register(r'api/comments', CommentViewSet)
router.register(r'api/profiles', ProfileViewSet)



from django.conf import settings

from .utils import get_notifications

def data_templates(request):
	'''
		context_processors for get in all templates 
	'''

	# Get notifications data
	notifications = get_notifications(request.user.id)

	return { 
		'SETTINGS': settings,
		'notifications': notifications
	}


# encoding:utf-8
from django import template
from django.contrib.contenttypes.models import ContentType
from django.utils import formats, timezone
from django.utils.text import Truncator

from hitcount.models import HitCount

from ..models import Comment, Forum, Topic, Notification
from .photo import get_photo
from ..utils import (
    get_photo_profile, get_datetime_topic
)

register = template.Library()


@register.filter
def in_category(category):
    '''
        This tag filter the forum for category
    '''
    return Forum.objects.filter(
        category_id=category,
        hidden=False
    )


@register.filter
def get_tot_comments(idtopic):
    '''
        This tag filter return the total
        comments of one topic
    '''
    return Comment.objects.filter(
        topic_id=idtopic,
    ).count()


@register.simple_tag
def get_tot_views(idtopic):
    '''
        This tag filter return the total
        views for topic or forum
    '''
    try:
        content = Topic.objects.get(idtopic=idtopic)
        idobj = ContentType.objects.get_for_model(content)

        hit = HitCount.objects.get(
            object_pk=idtopic,
            content_type_id=idobj
        )
        total = hit.hits
    except Exception:
        total = 0

    return total


@register.filter
def get_path_profile(user):
    '''
        Return tag a with profile
    '''
    username = getattr(user, "username")
    tag = "<a href='/profile/"+ username + "'>"+username+"</a>"

    return tag


@register.filter
def get_tot_users_comments(topic):
    '''
        This tag filter return the total
        users of one topic
    '''
    idtopic = topic.idtopic
    users = Comment.objects.filter(topic_id=idtopic)

    data = ""
    lista = []
    for user in users:
        usuario = user.user.username
        if not usuario in lista:
            lista.append(usuario)

            photo = get_photo(user.user.id)

            tooltip = "data-toggle='tooltip' data-placement='bottom' title='"+usuario+"'"
            data += "<a href='/profile/"+ usuario + "' "+tooltip+">"
            data += "<img class='img-circle' src='"+str(photo)+"' "
            data += "width=30, height=30></a>"

    if len(users) == 0:
        usuario = topic.user.username
        iduser = topic.user.id

        photo = get_photo(iduser)
        data += "<a href='/profile/"+ usuario + "'>"
        data += "<img class='img-circle' src='"+str(photo)+"' "
        data += "width=30, height=30></a>"

    return data


@register.filter
def get_tot_topics_moderate(forum):
    '''
        This filter return info about
        Few topics missing for moderate
    '''
    topics_count = forum.topics_count
    idforum = forum.idforum

    moderates = Topic.objects.filter(
        forum_id=idforum,
        moderate=True).count()
    return topics_count - moderates


@register.filter
def get_item_notification(notification):
    '''
        This filter return info about
        one notification of one user
    '''
    idobject = notification.idobject
    is_comment = notification.is_comment

    html = ""
    if is_comment:

        try:
            comment = Comment.objects.get(idcomment=idobject)
            forum = comment.topic.forum.name
            slug = comment.topic.slug
            idtopic = comment.topic.idtopic
            description = Truncator(comment.description).chars(100)
            description = description.replace("p>", "p class='black'>", 1)
            username = comment.user.username

            url_topic = "/topic/" + forum + "/" + \
                slug + "/" + str(idtopic) + "/"

            title = "<h5><a href='"+url_topic+"'><u>" + \
                comment.topic.title+"</u></h5></a>"

            description = "<p class='black'>"+description+"</p>"

            # Data profile
            photo = get_photo_profile(comment.user.id)
            user = "<a href='/profile/"+username + \
                "'><p>" + username + "</p></a>"

            date = get_datetime_topic(notification.date)

            # Notificacion
            html += '<div class="list-group">'
            html += '   <div class="list-group-item">'
            html += '      <div class="row-action-primary">'
            html += '           <img src="'+photo + \
                '" width=20 height=20 class="img-circle" />'
            html += '       </div>'
            html += '       <div class="row-content">'
            html += '           <h4 class="list-group-item-heading">' + \
                title.encode('utf8')+'</h4>'
            html += '           <p class="list-group-item-text">' + \
                description.encode('utf8')+'</p>'
            html += '           <p>'+user.encode('utf8')+'</p>'
            html += '           <p class="black">'+date+'</p>'
            html += '        </div>'
            html += '   </div>'
            html += '   <div class="list-group-separator"></div>'
            html += '</div>'

        except Comment.DoesNotExist:
            html = ""
    else:
        html = ""

    return html


@register.filter
def get_pending_notifications(user):
    '''
        This method return total pending notifications
    '''
    return Notification.objects.filter(
        is_view=False, iduser=user).count()


@register.filter
def get_last_activity(idtopic):
    '''
        This method return last activity of topic
    '''
    try:
        comment = Comment.objects.filter(
            topic_id=idtopic).order_by("-date")
    except Exception:
        comment = None

    if comment:
        # Get timezone for datetime
        d_timezone = timezone.localtime(comment[0].date)
        # Format data
        date = formats.date_format(d_timezone, "SHORT_DATETIME_FORMAT")

        # Return format data more user with tag <a>
        html = ""
        # html += get_path_profile(comment[0].user)
        html += " <p>"+str(date)+"</p>"
        return html
    else:
        topic = Topic.objects.get(idtopic=idtopic)

        # Get timezone for datetime
        d_timezone = timezone.localtime(topic.date)
        # Format data
        date = formats.date_format(d_timezone, "SHORT_DATETIME_FORMAT")

        # Return format data more user with tag <a>
        html = ""
        # html += get_path_profile(topic.user)
        html += " <p>"+str(date)+"</p>"
        return html
